# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-30 13:10
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailtrans', '0004_sitelanguages'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sitelanguages',
            name='other_languages',
            field=models.ManyToManyField(blank=True, to='wagtailtrans.Language'),
        ),
    ]
