# Generated by Django 2.0.3 on 2018-03-27 09:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailtrans', '0006_auto_20161212_2020'),
    ]

    operations = [
        migrations.AlterField(
            model_name='language',
            name='is_default',
            field=models.BooleanField(default=False, help_text='Visitors with no language preference will see the site in this language'),
        ),
        migrations.AlterField(
            model_name='language',
            name='position',
            field=models.IntegerField(default=0, help_text='Language choices and translations will be displayed in this order'),
        ),
        migrations.AlterField(
            model_name='sitelanguages',
            name='default_language',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='site_default_language', to='wagtailtrans.Language'),
        ),
    ]
