# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-21 12:19
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0029_unicode_slugfield_dj19'),
        ('wagtailtrans', '0003_auto_20161121_1211'),
    ]

    operations = [
        migrations.CreateModel(
            name='SiteLanguages',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('default_language', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='site_default_language', to='wagtailtrans.Language')),
                ('other_languages', models.ManyToManyField(to='wagtailtrans.Language')),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
            options={
                'verbose_name': 'Site languages',
                'verbose_name_plural': 'Site languages',
            },
        ),
    ]
