# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-12-12 20:20
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0029_unicode_slugfield_dj19'),
        ('wagtailtrans', '0005_auto_20161130_1310'),
    ]

    operations = [
        migrations.RenameField(
            model_name='translatablepage',
            old_name='page_ptr',
            new_name='translatable_page_ptr',
        ),
        migrations.AlterField(
            model_name='translatablepage',
            name='translatable_page_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='wagtailcore.Page'),
        ),
    ]
