# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-11-06 12:31
from django.db import migrations, models
import django.db.models.deletion
import wagtailtrans.models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailtrans', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='language',
            name='code',
            field=models.CharField(max_length=12, unique=True),
        ),
    ]
